# -*- coding: utf-8 -*-
#
# Copyright 2021 Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service fixtures for integration testing."""
import contextlib
import json
import os
import shutil
import uuid
from copy import deepcopy

import pytest
from git import GitCommandError, Repo


@contextlib.contextmanager
def _mock_cache_sync(repo):
    """Mocks the resetting of the cache since other fixtures perform migrations on the cache without pushing.

    We don't want to undo that temporary migration with an actual cache sync, as it would break tests with
    repeat service calls, if the migration was just done locally in the fixture.
    """
    from renku.service.controllers.api import mixins

    current_head = repo.head.ref

    def _mocked_repo_reset(self, project):
        """Mock repo reset to work with mocked renku save."""
        repo.git.reset("--hard", current_head)

    reset_repo_function = mixins.ReadOperationMixin.reset_local_repo
    mixins.ReadOperationMixin.reset_local_repo = _mocked_repo_reset

    try:
        yield
    finally:
        mixins.ReadOperationMixin.reset_local_repo = reset_repo_function


def integration_repo_path(headers, project_id, url_components):
    """Constructs integration repo path."""
    from renku.service.serializers.headers import RequiredIdentityHeaders
    from renku.service.utils import make_project_path

    user = RequiredIdentityHeaders().load(headers)
    project = {
        "project_id": project_id,
        "owner": url_components.owner,
        "name": url_components.name,
    }

    project_path = make_project_path(user, project)
    return project_path


@contextlib.contextmanager
def integration_repo(headers, project_id, url_components):
    """With integration repo helper."""
    from git import Repo

    from renku.core.utils.contexts import chdir

    repo_path = integration_repo_path(headers, project_id, url_components)
    with chdir(repo_path):
        repo = Repo(repo_path)
        repo.heads.master.checkout()

        yield repo

        if integration_repo_path(headers, project_id, url_components).exists():
            repo.git.reset("--hard")
            repo.heads.master.checkout()
            repo.git.reset("--hard")
            repo.git.clean("-xdf")


@pytest.fixture(scope="module")
def integration_lifecycle(svc_client, mock_redis, identity_headers, it_remote_repo_url):
    """Setup and teardown steps for integration tests."""
    from renku.core.models.git import GitURL

    url_components = GitURL.parse(it_remote_repo_url)

    payload = {"git_url": it_remote_repo_url, "depth": 0}

    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers,)

    assert response
    assert {"result"} == set(response.json.keys())

    project_id = response.json["result"]["project_id"]
    assert isinstance(uuid.UUID(project_id), uuid.UUID)

    yield svc_client, identity_headers, project_id, url_components

    # Teardown step: Delete all branches except master (if needed).
    if integration_repo_path(identity_headers, project_id, url_components).exists():
        with integration_repo(identity_headers, project_id, url_components) as repo:
            try:
                repo.remote().push(refspec=(":{0}".format(repo.active_branch.name)))
            except GitCommandError:
                pass


@pytest.fixture
def svc_client_setup(integration_lifecycle):
    """Service client setup."""
    svc_client, headers, project_id, url_components = integration_lifecycle

    with integration_repo(headers, project_id, url_components) as repo:
        repo.git.checkout("master")

        new_branch = uuid.uuid4().hex
        current = repo.create_head(new_branch)
        current.checkout()

        with _mock_cache_sync(repo):
            yield svc_client, deepcopy(headers), project_id, url_components, repo

        if integration_repo_path(headers, project_id, url_components).exists():
            # NOTE: Some tests delete the repo
            repo.git.checkout("master")
            repo.git.branch("-D", current)


@pytest.fixture
def svc_client_with_repo(svc_client_setup):
    """Service client with a remote repository."""
    svc_client, headers, project_id, url_components, repo = svc_client_setup

    response = svc_client.post(
        "/cache.migrate", data=json.dumps(dict(project_id=project_id, skip_docker_update=True)), headers=headers
    )
    assert response.json["result"]

    with _mock_cache_sync(repo):
        yield svc_client, deepcopy(headers), project_id, url_components


@pytest.fixture
def svc_protected_repo(svc_client, identity_headers, it_protected_repo_url):
    """Service client with migrated remote protected repository."""
    from renku.core.models.git import GitURL

    payload = {
        "git_url": it_protected_repo_url,
        "depth": 0,
    }

    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)

    data = {
        "project_id": response.json["result"]["project_id"],
        "skip_template_update": True,
        "skip_docker_update": True,
    }
    svc_client.post("/cache.migrate", data=json.dumps(data), headers=identity_headers)

    url_components = GitURL.parse(it_protected_repo_url)

    with integration_repo(identity_headers, response.json["result"]["project_id"], url_components) as repo:
        with _mock_cache_sync(repo):
            yield svc_client, identity_headers, payload, response


@pytest.fixture
def svc_protected_old_repo(svc_synced_client, it_protected_repo_url):
    """Service client with remote protected repository."""
    svc_client, identity_headers, cache, user = svc_synced_client

    payload = {
        "git_url": it_protected_repo_url,
        "depth": 0,
    }

    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
    project_id = response.json["result"]["project_id"]

    yield svc_client, identity_headers, project_id, cache, user


@pytest.fixture()
def local_remote_repository(svc_client, tmp_path, mock_redis, identity_headers, real_sync):
    """Client with a local remote to test pushes."""
    from click.testing import CliRunner
    from git.config import GitConfigParser, get_config_path
    from marshmallow import post_load, pre_load

    from renku.cli import cli
    from renku.core.utils.contexts import chdir
    from renku.service.serializers import cache

    # NOTE: prevent service from adding an auth token as it doesn't work with local repos
    def _no_auth_format(self, data, **kwargs):
        data["url_with_auth"] = data["git_url"]
        return data

    orig_format_url = cache.ProjectCloneContext.format_url

    cache.ProjectCloneContext.format_url = post_load(_no_auth_format)

    # NOTE: mock owner/project so service is happy
    def _mock_owner(self, data, **kwargs):
        data["owner"] = "dummy"

        data["name"] = "project"

        return data

    orig_set_owner = cache.ProjectCloneContext.set_owner_name

    cache.ProjectCloneContext.set_owner_name = pre_load(_mock_owner)

    remote_repo_path = tmp_path / "remote_repo"
    remote_repo_path.mkdir()

    remote_repo = Repo.init(remote_repo_path, bare=True)

    remote_repo_checkout_path = tmp_path / "remote_repo_checkout"
    remote_repo_checkout_path.mkdir()

    remote_repo_checkout = remote_repo.clone(str(remote_repo_checkout_path))

    home = tmp_path / "user_home"
    home.mkdir()
    old_home = os.environ.get("HOME", "")
    old_xdg_home = os.environ.get("XDG_CONFIG_HOME", "")

    try:
        # NOTE: fake user home directory
        os.environ["HOME"] = str(home)
        os.environ["XDG_CONFIG_HOME"] = str(home)

        with GitConfigParser(get_config_path("global"), read_only=False) as global_config:
            global_config.set_value("user", "name", "Renku @ SDSC")
            global_config.set_value("user", "email", "renku@datascience.ch")

        # NOTE: init "remote" repo
        runner = CliRunner()
        with chdir(remote_repo_checkout_path):

            result = runner.invoke(
                cli, ["init", ".", "--template-id", "python-minimal", "--force"], "\n", catch_exceptions=False
            )
            assert 0 == result.exit_code

            remote_name = remote_repo_checkout.active_branch.tracking_branch().remote_name
            remote = remote_repo_checkout.remotes[remote_name]
            result = remote.push()
    finally:
        os.environ["HOME"] = old_home
        os.environ["XDG_CONFIG_HOME"] = old_xdg_home
        try:
            shutil.rmtree(home)
        except OSError:  # noqa: B014
            pass

        payload = {"git_url": f"file://{remote_repo_path}", "depth": 0}

        response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers,)

        assert response
        assert {"result"} == set(response.json.keys())

        project_id = response.json["result"]["project_id"]
        assert isinstance(uuid.UUID(project_id), uuid.UUID)

    try:
        yield svc_client, identity_headers, project_id, remote_repo, remote_repo_checkout
    finally:
        cache.ProjectCloneContext.format_url = orig_format_url
        cache.ProjectCloneContext.set_owner_name = orig_set_owner

        try:
            shutil.rmtree(remote_repo_path)
        except OSError:  # noqa: B014
            pass

        try:
            shutil.rmtree(remote_repo_checkout_path)
        except OSError:  # noqa: B014
            pass



# -*- coding: utf-8 -*-
#
# Copyright 2021 Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku core fixtures for datasets testing."""
import pytest


@pytest.fixture
def dataset_responses():
    """Authentication responses."""
    import responses

    with responses.RequestsMock(assert_all_requests_are_fired=False) as response:

        def request_callback(request):
            return 200, {"Content-Type": "application/text"}, "123"

        response.add_callback(responses.GET, "http://example.com/file1", callback=request_callback)
        response.add_callback(responses.GET, "https://example.com/file1", callback=request_callback)
        response.add_callback(responses.GET, "http://example.com/file1.ext?foo=bar", callback=request_callback)
        response.add_callback(responses.HEAD, "http://example.com/file1", callback=request_callback)
        response.add_callback(responses.HEAD, "https://example.com/file1", callback=request_callback)
        response.add_callback(responses.HEAD, "http://example.com/file1.ext?foo=bar", callback=request_callback)

        yield response


@pytest.fixture
def client_with_datasets(client, directory_tree):
    """A client with datasets."""
    from renku.core.models.provenance.agents import Person

    person_1 = Person.from_string("P1 <p1@example.com> [IANA]")
    person_2 = Person.from_string("P2 <p2@example.com>")

    client.create_dataset(name="dataset-1", keywords=["dataset", "1"], creators=[person_1])

    with client.with_dataset("dataset-2", create=True) as dataset:
        dataset.keywords = ["dataset", "2"]
        dataset.creators = [person_1, person_2]

        client.add_data_to_dataset(dataset=dataset, urls=[str(p) for p in directory_tree.glob("*")])

    yield client



# -*- coding: utf-8 -*-
#
# Copyright 2018-2021- Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku show command."""

from renku.core.commands.graph import Graph
from renku.core.incubation.command import Command
from renku.core.utils import communication


def get_status():
    """Show a status of the repository."""
    return Command().command(_get_status).require_migration().require_clean()


def _get_status(client, revision, no_output, path):
    graph = Graph(client)
    # TODO filter only paths = {graph.normalize_path(p) for p in path}
    status = graph.build_status(revision=revision, can_be_cwl=no_output)

    if client.has_external_files():
        communication.echo(
            "Changes in external files are not detected automatically. To "
            'update external files run "renku dataset update -e".'
        )

    try:
        communication.echo("On branch {0}".format(client.repo.active_branch))
    except TypeError:
        communication.error("Git HEAD is detached!\n" " Please move back to your working branch to use renku\n")

    return graph, status



# -*- coding: utf-8 -*-
#
# Copyright 2021 Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku common configurations."""
import subprocess
import time

import pytest


@pytest.fixture()
def run_shell():
    """Create a shell cmd runner."""

    def run_(cmd, return_ps=None, sleep_for=None):
        """Spawn subprocess and execute shell command.

        :param return_ps: Return process object.
        :param sleep_for: After executing command sleep for n seconds.
        :returns: Process object or tuple (stdout, stderr).
        """
        ps = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

        if return_ps:
            return ps

        output = ps.communicate()

        if sleep_for:
            time.sleep(sleep_for)

        return output

    return run_


@pytest.fixture()
def runner():
    """Create a runner on isolated filesystem."""
    from click.testing import CliRunner

    return CliRunner()


@pytest.fixture()
def run(runner, capsys):
    """Return a callable runner."""
    from renku.cli import cli
    from renku.core.utils.contexts import Isolation

    def generate(args=("update", "--all",), cwd=None, **streams):
        """Generate an output."""
        with capsys.disabled(), Isolation(cwd=cwd, **streams):
            try:
                cli.main(
                    args=args, prog_name=runner.get_default_prog_name(cli),
                )
                return 0
            except SystemExit as e:
                return 0 if e.code is None else e.code
            except Exception:
                raise

    return generate


@pytest.fixture()
def isolated_runner():
    """Create a runner on isolated filesystem."""
    from click.testing import CliRunner

    runner_ = CliRunner()
    with runner_.isolated_filesystem():
        yield runner_



# -*- coding: utf-8 -*-
#
# Copyright 2021 Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku core fixtures for project management."""
import os
import tempfile

import pytest


@pytest.fixture
def local_client():
    """Add a Renku local client."""
    from renku.core.management import LocalClient

    with tempfile.TemporaryDirectory() as tempdir:
        yield LocalClient(path=tempdir)


@pytest.fixture
def reset_environment(svc_client, mock_redis):
    """Restore environment variable to their values before test execution."""
    current_environment = os.environ.copy()

    try:
        yield
    finally:
        os.environ.clear()
        os.environ.update(current_environment)



# -*- coding: utf-8 -*-
#
# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Migrate project to the latest Renku version."""
import json
import os

import click

from renku.cli.utils.callback import ClickCallback
from renku.core.commands.echo import WARNING
from renku.core.commands.migrate import (
    MIGRATION_REQUIRED,
    NON_RENKU_REPOSITORY,
    UNSUPPORTED_PROJECT,
    check_immutable_template_files_command,
    check_project,
    migrate_project,
    migrations_check,
    migrations_versions,
)
from renku.core.errors import MigrationRequired, ProjectNotSupported


@click.command()
@click.option("-c", "--check", is_flag=True, help="Check if migration is required and quit.")
@click.option("--no-commit", is_flag=True, hidden=True, help="Do not commit changes after the migration.")
def migrate(check, no_commit):
    """Check for migration and migrate to the latest Renku project version."""
    if check:
        status = check_project().build().execute().output
        if status == MIGRATION_REQUIRED:
            raise MigrationRequired
        elif status == UNSUPPORTED_PROJECT:
            raise ProjectNotSupported
        elif status == NON_RENKU_REPOSITORY:
            click.secho(WARNING + "Not a renku project.")

        return

    communicator = ClickCallback()

    command = migrate_project().with_communicator(communicator)
    if not no_commit:
        command = command.with_commit()
    result = command.build().execute(skip_template_update=True, skip_docker_update=True)

    result, _, _ = result.output

    if result:
        click.secho("OK", fg="green")
    else:
        if check_project().build().execute().output == NON_RENKU_REPOSITORY:
            click.secho(WARNING + "Not a renku project.")
        click.secho("No migrations required.")


@click.command(hidden=True)
def migrationscheck():
    """Check status of the project and current renku-python version."""
    latest_version, project_version = migrations_versions().build().execute().output
    (
        migration_required,
        project_supported,
        template_update_possible,
        current_template_version,
        latest_template_version,
        template_source,
        template_ref,
        template_id,
        automated_update,
        docker_update_possible,
    ) = (migrations_check().build().execute().output)

    click.echo(
        json.dumps(
            {
                "latest_version": latest_version,
                "project_version": project_version,
                "migration_required": migration_required,
                "project_supported": project_supported,
                "template_update_possible": template_update_possible,
                "current_template_version": str(current_template_version),
                "latest_template_version": str(latest_template_version),
                "template_source": template_source,
                "template_ref": template_ref,
                "template_id": template_id,
                "automated_update": automated_update,
                "docker_update_possible": docker_update_possible,
            }
        )
    )


@click.command(hidden=True)
@click.argument(
    "paths", type=click.Path(exists=True, dir_okay=True), nargs=-1, required=True,
)
def check_immutable_template_files(paths):
    """Check specified paths if they are marked immutable in the template."""
    result = check_immutable_template_files_command().build().execute(paths=paths)
    paths = result.output
    if paths:
        click.echo(os.linesep.join(paths))
        exit(1)



# -*- coding: utf-8 -*-
#
# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Git utility functions."""

import math
from subprocess import SubprocessError, run

from renku.core import errors

ARGUMENT_BATCH_SIZE = 100


def run_command(command, *paths, separator=None, **kwargs):
    """Execute command by splitting `paths` to make sure that argument list will be within os limits.

    :param command: A list or tuple containing command and its arguments.
    :param paths: List of paths/long argument. This will be appended to `command` for each invocation.
    :param separator: Separator for `paths` if they need to be passed as string.
    :param kwargs: Extra arguments passed to `subprocess.run`.
    :returns: Result of last invocation.
    """
    result = None

    for batch in split_paths(*paths):
        if separator:
            batch = [separator.join(batch)]

        try:
            if not isinstance(batch, list):
                batch = list(batch)
            result = run(command + batch, **kwargs)

            if result.returncode != 0:
                break
        except KeyboardInterrupt:
            raise
        except SubprocessError as e:
            raise errors.GitError(f"Cannot run command {command} : {e}")

    return result


def add_to_git(git, *paths, **kwargs):
    """Split `paths` and add them to git to make sure that argument list will be within os limits."""
    for batch in split_paths(*paths):
        git.add(*batch, **kwargs)


def split_paths(*paths):
    """Return a generator with split list of paths."""
    batch_count = math.ceil(len(paths) / ARGUMENT_BATCH_SIZE)
    batch_count = max(batch_count, 1)

    for index in range(batch_count):
        yield paths[index * ARGUMENT_BATCH_SIZE : (index + 1) * ARGUMENT_BATCH_SIZE]



# -*- coding: utf-8 -*-
#
# Copyright 2019-2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service graph jobs tests."""
import json

import pytest

from renku.service.jobs.graph import graph_build_job


@pytest.mark.service
@pytest.mark.jobs
@pytest.mark.integration
def test_graph_build_job(it_remote_repo_url, it_git_access_token):
    """Test graph build job."""
    payload = {
        "git_url": it_remote_repo_url,
        "revision": "HEAD",
        "callback_url": "https://webhook.site",
        "token": it_git_access_token,
    }

    data = graph_build_job(**payload)
    assert {"project_url", "commit_id", "payload"} == set(data.keys())

    json_ld = json.loads(data["payload"])
    assert json_ld


@pytest.mark.service
@pytest.mark.jobs
@pytest.mark.integration
def test_graph_build_job_no_callback(it_remote_repo_url, it_git_access_token):
    """Test graph build job with missing callback."""
    payload = {"git_url": it_remote_repo_url, "revision": "HEAD", "callback_url": "", "token": it_git_access_token}

    with pytest.raises(RuntimeError):
        graph_build_job(**payload)


@pytest.mark.service
@pytest.mark.jobs
@pytest.mark.integration
def test_graph_build_job_no_revision(it_remote_repo_url, it_git_access_token):
    """Test graph build job with missing revision."""
    payload = {
        "git_url": it_remote_repo_url,
        "revision": None,
        "callback_url": "https://webhook.site",
        "token": it_git_access_token,
    }

    data = graph_build_job(**payload)
    assert {"project_url", "commit_id", "payload"} == set(data.keys())

    json_ld = json.loads(data["payload"])
    assert json_ld


@pytest.mark.service
@pytest.mark.jobs
@pytest.mark.integration
def test_graph_build_job_git_url(it_git_access_token):
    """Test graph build job with missing git url."""
    payload = {"git_url": None, "revision": "HEAD", "callback_url": "", "token": it_git_access_token}

    with pytest.raises(RuntimeError):
        graph_build_job(**payload)


@pytest.mark.service
@pytest.mark.jobs
@pytest.mark.integration
def test_graph_build_job_missing_token(it_remote_repo_url):
    """Test graph build job with missing token."""
    payload = {"git_url": it_remote_repo_url, "revision": "HEAD", "callback_url": "", "token": None}

    with pytest.raises(RuntimeError):
        graph_build_job(**payload)


@pytest.mark.service
@pytest.mark.jobs
@pytest.mark.integration
def test_graph_build_job_invalid_token(it_remote_repo_url):
    """Test graph build job with wrong token."""
    payload = {"git_url": it_remote_repo_url, "revision": "HEAD", "callback_url": "", "token": "123"}

    with pytest.raises(RuntimeError):
        graph_build_job(**payload)



# -*- coding: utf-8 -*-
#
# Copyright 2021 Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service fixtures for project management."""
import uuid
from pathlib import Path

import pytest


@pytest.fixture
def project_metadata(project):
    """Create project with metadata."""
    metadata = {
        "project_id": uuid.uuid4().hex,
        "name": Path(project).name,
        "fullname": "full project name",
        "email": "my@email.com",
        "owner": "me",
        "token": "awesome token",
        "git_url": "git@gitlab.com",
        "initialized": True,
    }

    yield project, metadata


@pytest.fixture(scope="module")
def it_git_access_token():
    """Returns a git access token for a testing run."""
    from tests.fixtures.config import IT_GIT_ACCESS_TOKEN

    return IT_GIT_ACCESS_TOKEN


@pytest.fixture(scope="module")
def it_remote_repo_url():
    """Returns a remote path to integration test repository."""
    from tests.fixtures.config import IT_REMOTE_REPO_URL

    return IT_REMOTE_REPO_URL


@pytest.fixture(scope="module")
def it_non_renku_repo_url():
    """Returns a remote path to integration test repository."""
    from tests.fixtures.config import IT_REMOTE_NON_RENKU_REPO_URL

    return IT_REMOTE_NON_RENKU_REPO_URL


@pytest.fixture(scope="module")
def it_no_commit_repo_url():
    """Returns a remote path to integration test repository."""
    from tests.fixtures.config import IT_REMOTE_NO_COMMITS_REPO_URL

    return IT_REMOTE_NO_COMMITS_REPO_URL


@pytest.fixture(scope="module")
def it_protected_repo_url():
    """Returns a protected repository url."""
    from tests.fixtures.config import IT_PROTECTED_REMOTE_REPO_URL

    return IT_PROTECTED_REMOTE_REPO_URL
